/*
 * File adoption routine 
 *
 */
#include <errno.h>
#include <time.h>
#include <ctype.h>
#include <dirent.h>
#include <getopt.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/vfs.h>

#include "pimppa.h"
#include "adopt.h"
#include "news.h"
#include "dupecheck.h"
#include "md5.h"

extern MYSQL src_db, dst_db;

/*
 * Checks directory "path" and deletes all files from it which
 * are found to be duplicates or otherwise unsuitable. Adds
 * files to the filearea.
 *
 */
int p_adopt(char *path, int default_area, unsigned long options)
{
	char olddir[PATH_MAX];
	MYSQL_RES *sql_res;
	MYSQL_ROW sql_row;
	DIR *dirp;
	struct dirent *dirdata;
	int gained_files=0,gained_bytes=0;
	int deleted_files=0,deleted_bytes=0;
	FILE *fp;
	int nazi=0,verbose=0, domove=0;
	int strict_md5;
	int to_lowercase,minassnamelen;
	char *value;

	if(options & OPT_NAZI)
		nazi=1;
	if(options & OPT_VERBOSE)
		verbose=1;
	if(options & OPT_MOVE)
		domove=1;

	value=p_getmisc(&src_db, P_KEY_STRICTMD5);
	if(value)
		strict_md5=atoi(value);
	else
		strict_md5=atoi(P_STRICTMD5);

	value=p_getmisc(&src_db, P_KEY_TOLOWERCASE);
	if(value)
		to_lowercase=atoi(value);
	else
		to_lowercase=atoi(P_TOLOWERCASE);

	value=p_getmisc(&src_db, P_KEY_MINASSNAMELEN);
	if(value)
		minassnamelen=atoi(value);
	else
		minassnamelen=atoi(P_MIN_ASS_NAMELENGTH);
		
	dirp=opendir(path);
	if(!dirp)
	{
		fprintf(stderr, "Unable to open %s\n", path);
		return(0);
	}

	getcwd(olddir, PATH_MAX);
	chdir(path);

	while((dirdata=readdir(dirp)))
	{
		char fullpath[PATH_MAX];
		int new_file;
		char *filename=dirdata->d_name;
		struct stat st;
		char escaped_fn[2*P_LEN_FILE_NAME+1];
		char md5sum[16];
		char md5sum_esc[16*2+1];
		int destarea_id;
		int file_integ=0;

		new_file=1;
		destarea_id=default_area;
		
		if(dirdata->d_reclen>=P_LEN_FILE_NAME)	// Oops, too long filename
		{
			if(verbose)
				printf("%s\n  !! Over %d chars filename, deleted\n",
						dirdata->d_name, P_LEN_FILE_NAME);

			unlink(dirdata->d_name);
			deleted_files++;
			deleted_bytes+=st.st_size;
			continue;
		}
		
		sprintf(fullpath, "%s/%s", path, filename);

		stat(fullpath, &st);
		if(!(S_ISREG(st.st_mode) || S_ISLNK(st.st_mode))
			|| S_ISDIR(st.st_mode))
			continue;
		
		if(to_lowercase)				// Convert filename
		{
			char newpath[PATH_MAX];

			p_strtolc(filename);
					
			sprintf(newpath, "%s/%s", path, filename);
			rename(fullpath, newpath);
			
			strcpy(fullpath, newpath);
		}
		
		if(verbose && domove)
			printf("%s\n", filename);

//		printf("at 1\n");
/************ Dupecheck the filename *************************/
	
		file_integ=p_dupecheck(filename, destarea_id);
		if(file_integ>=0 && file_integ!=INTEG_FAILED)	// Exists, not failed
		{
			if(domove)				// We're moving, but dupe, delete
			{
				unlink(fullpath);
				deleted_files++;
				deleted_bytes+=st.st_size;
			
				if(verbose)
					printf("  !! Exists, deleted\n");
			}
			// else don't readopt an existing file

			continue;
		}
		// After this point, the passed file is either missing
		// from database or broken (INTEG_FAILED)
		
//		printf("at 2\n");

/********** Is the filename valid at all ***********/

		if(!p_checkfilename(&dst_db, filename))
		{
			if(verbose)
			{
				if(!domove)
					printf("%s\n", filename);
				
				printf("  !! Invalid filename, deleted\n");
			}

			unlink(fullpath);
			deleted_files++;
			deleted_bytes+=st.st_size;
			continue;
		}
		
//		printf("at 3\n");
	
/************ md5sum calc & dupecheck ***************/

		// Calculate md5sum
		memset(md5sum, 0, 16);
		fp=fopen(fullpath, "rb");
		if(fp)
		{
			if(md5_stream(fp, md5sum))
				fprintf(stderr, "md5_stream() fail on %s\n", fullpath);
			
			fclose(fp);
		}
		
//		printf("at 3a\n");

		if(strict_md5 && p_md5check(md5sum))
		{
			// Do not delete existing file thats broken, its just the same file
			if(!domove && file_integ>=0)
				continue;
		
//			printf("at 3c\n");
			
			unlink(fullpath);
			deleted_files++;
			deleted_bytes+=st.st_size;
			
			if(verbose)
			{
				int cnt;
				
				if(!domove)
					printf("%s\n", filename);
				
				printf("  !! md5sum ");
				for(cnt=0;cnt<16;cnt++)
					printf("%02x", (unsigned char)md5sum[cnt]);
				printf(" collided, deleted\n");
				
				continue;
			}
		}
			
		mysql_escape_string(md5sum_esc, md5sum, 16);
		
//		printf("at 4\n");

/**** If we're moving, find the destination area *************/

		if(domove)
		{
			destarea_id=p_getdest(filename);
			if(destarea_id==0)						// It didn't know :(
				destarea_id=default_area;

			if(verbose)
				printf("  => %d", destarea_id);

			// Check the typical target areas. If destarea_id is not
			// among them, reset it to default_area
			if(destarea_id>0 && destarea_id!=default_area)
			{
				p_query(&src_db, "SELECT 
								  (%d REGEXP area_targets)
								  FROM p_areas
			   			          WHERE area_id=%d",
							     destarea_id, default_area);
				sql_res=mysql_store_result(&src_db);
				if(sql_res)	
				{
					if((sql_row=mysql_fetch_row(sql_res))) 
					{
						if(atoi(sql_row[0])<=0)
						{
							if(verbose)
								printf("	!! Resetting target area %d to default %d\n", destarea_id, default_area);
							destarea_id=default_area;
						}
					}
					mysql_free_result(sql_res);
				}
			}
	
			if(verbose && destarea_id==-1)
				printf("  !! Assign patterns ordered autokill\n");
	
			if(destarea_id==default_area && nazi)	// Unknown file, delete!
			{ 
				if(verbose)
					printf("  !! NaziSays: Delete!\n");
	
				destarea_id=-1;
			}

			if(destarea_id<0)					// Marked for deletion?
			{	
				unlink(fullpath);
				deleted_files++;
				deleted_bytes+=st.st_size;
				continue;
			}
		
//			printf("at 5\n");
		
/******* Locate destination area path and move file **************/

			p_query(&src_db, "SELECT area_path, area_flags FROM p_areas
							WHERE area_id=%d",
							destarea_id);
			sql_res=mysql_store_result(&src_db);
			if(sql_res)
			{
				if((sql_row=mysql_fetch_row(sql_res)))	// Ok, move!
				{
					char newpath[PATH_MAX];
		
					sprintf(newpath, "%s%s%s", 
							sql_row[0], 
							(p_checkp(sql_row[0]) ? "" : "/"),
							filename);
			
					if(!p_mv(fullpath, newpath))
					{
						fprintf(stderr, "p_mv() error from %s to %s\n",
								fullpath, newpath);
					}
							
					strcpy(fullpath, newpath);
					
					if(verbose)
						printf("  => %s\n", sql_row[0]);
				}
				mysql_free_result(sql_res);
			}
		}
		else
		{
			// not moving, we're already on some pimppa filearea
			// Lets create assign pattern
			
//			printf("at 6\n");
		
			p_assign(&dst_db, filename, minassnamelen, destarea_id);
		}
			
//		printf("at 7\n");

/******  All seems well. Lets punch the file in *********/

		gained_bytes+=st.st_size;
		gained_files++;

		mysql_escape_string(escaped_fn, filename, strlen(filename));
		
		p_query(&src_db, "REPLACE INTO p_files (file_name, file_size,
						file_area,file_date,file_backup, file_md5sum,
						file_integ,file_trans)
						VALUES ('%s',%ld,%d,FROM_UNIXTIME(%ld),0,'%s',%d,%d)",
						escaped_fn, st.st_size, destarea_id, 
						(long) time(NULL), md5sum_esc, 
						INTEG_NEW,TRANS_NEW);
		
//		printf("at 8\n");

		if(domove && verbose)
			printf("  == OK\n");
		fflush(stdout);
	}

	closedir(dirp);
	chdir(olddir);

	if(options & OPT_STATS)
	{
		printf("Ok, %d files and %d bytes gained, %d files and %d bytes were deleted.\n",
				gained_files,gained_bytes,
				deleted_files, deleted_bytes);
	}

	fflush(stdout);
	return(gained_files);
}

