/*
 * PIMPPA duplicate checking routines
 *
 *
 */

#include <errno.h>
#include <time.h>
#include <ctype.h>
#include <dirent.h>
#include <getopt.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/vfs.h>

#include "pimppa.h"
#include "news.h"
#include "dupecheck.h"

extern MYSQL src_db;

int getlastcomp(char *string);
int dupecheck_single(char *escaped_fn, int area_id);
int dupecheck_regexp(char *escaped_fn, char *regexp);
int dupecheck(char *filename, int area_id);

/*
 * Returns the last component after a dot in a string with the format
 *
 * blaa.bloo.blii.bluup  == bluup 
 *
 */
int getlastcomp(char *string)
{
	int i;

	i=strlen(string);
	while(i>0 && string[i]!='.')
		i--;

	if(i>0)
		return(i+1);
	else
		return(0);
}

/************ Dupecheck for future transformed filename *********/


/*
 * Returns file integrity value if file exists on "area_id", 
 * or on any area that is defined in p_areas -table as that 
 * areas immediate target areas.
 *
 * -1 is returned if the file was not found
 *
 */
int p_dupecheck(char *filename, int area_id)
{
	MYSQL_ROW sql_row;
	MYSQL_RES *sql_res;
	int retval=-1;

	char transname[P_LEN_FILE_NAME+1];
	char *transptr;

/***** Check normal filename ****************/
	
	retval=dupecheck(filename, area_id);
	if(retval>=0)
		return(retval);

/***** Dupecheck transformed filename *******/

	strcpy(transname, filename);
	transptr=strrchr(transname, '.');		// Next char from '.'
	if(transptr)
	{
		char ext[PATH_MAX];
		char escaped_transn[2*P_LEN_FILE_NAME+1];

		transptr++;
		ext[0]=0;
		
		mysql_escape_string(escaped_transn, transptr, strlen(transptr));

		p_query(&src_db, "SELECT type_transto
						FROM p_types
						WHERE type_ext='%s'
						AND type_transcmd<>''
						AND type_transto<>''",
						escaped_transn);
		if(mysql_error(&src_db)[0])
		{
			fprintf(stderr, "Err: %s\n", mysql_error(&src_db));
			return(retval);
		}
		sql_res=mysql_store_result(&src_db);
		if(sql_res)
		{
			if((sql_row=mysql_fetch_row(sql_res)))
				strcpy(ext, sql_row[0]);
				
			mysql_free_result(sql_res);
		}

		if(ext[0])
		{
			strcpy(transptr, ext);				// Attach new extension
	
			if(strcmp(transname, filename)!=0)  // Name changed?
			{
//				printf("	=> %s\n", transname);

				retval=dupecheck(transname, area_id);
				if(retval>=0)								// Exists
					return(retval);
			}
		}
	}

	return(retval);
}

int dupecheck(char *filename, int area_id)
{
	MYSQL_RES *sql_res;
	MYSQL_ROW sql_row;
	char escaped_fn[PATH_MAX];
	int retval=-1;

	mysql_escape_string(escaped_fn, filename, strlen(filename));

	if(area_id==-1)						// Check from all areas
	{
//		printf("	DEBUG: Dupechecking %s from all...\n", filename);

		p_query(&src_db, "SELECT file_integ FROM p_files 
						WHERE file_name='%s'", 
						escaped_fn);
		sql_res=mysql_store_result(&src_db);
		if(sql_res)
		{
			if(mysql_num_rows(sql_res)>0)		// Exists
			{
				sql_row=mysql_fetch_row(sql_res);
				retval=atoi(sql_row[0]);
				mysql_free_result(sql_res);
				
				return(retval);
			}

			mysql_free_result(sql_res);
		}
		return(retval);
	}

/************ Specific area and its targets *****************/

	retval=dupecheck_single(escaped_fn, area_id);
	if(retval>=0)
		return(retval);

	p_query(&src_db, "SELECT area_targets FROM p_areas
		              WHERE area_id=%d AND (area_flags & %ld)",
				      area_id, AREA_INCOMING);
	sql_res=mysql_store_result(&src_db);
	if(sql_res)	
	{
		if(mysql_num_rows(sql_res)<=0)
		{
			mysql_free_result(sql_res);
			return(retval);
		}
		
		sql_row=mysql_fetch_row(sql_res);

		retval=dupecheck_regexp(escaped_fn, sql_row[0]);
			
		mysql_free_result(sql_res);
		
		if(retval>=0)
			return(retval);
	}

	return(retval);
}

/*
 * Dupechecks from a single area, no target areas noted
 *
 */
int dupecheck_single(char *escaped_fn, int area_id)
{
	MYSQL_RES *sql_res;
	MYSQL_ROW sql_row;
	int retval=-1;

//	printf("	DEBUG: Dupechecking %s from %d\n", escaped_fn, area_id);

//	printf("SELECT 1 FROM p_files
	p_query(&src_db, "SELECT file_integ FROM p_files
		              WHERE file_name='%s' AND file_area=%d",
				      escaped_fn, area_id);
	sql_res=mysql_store_result(&src_db);
	if(sql_res)	
	{
		if(mysql_num_rows(sql_res)>0)		// Exists
		{
			sql_row=mysql_fetch_row(sql_res);
			retval=atoi(sql_row[0]);
			
			mysql_free_result(sql_res);
			
			return(retval);
		}
		mysql_free_result(sql_res);
	}

	return(retval);
}

/*
 * Dupechecks from regexp areas
 *
 */
int dupecheck_regexp(char *escaped_fn, char *regexp)
{
	MYSQL_RES *sql_res;
	MYSQL_ROW sql_row;
	int retval=-1;

//	printf("	DEBUG: RegExpDChecking %s from %s\n", escaped_fn, regexp);

	p_query(&src_db, "SELECT file_integ FROM p_files
		              WHERE file_name='%s'
					  AND file_area RLIKE '%s'",
				      escaped_fn, regexp);
	sql_res=mysql_store_result(&src_db);
	if(sql_res)	
	{
		if(mysql_num_rows(sql_res)<=0)		// Exists
		{
			mysql_free_result(sql_res);
			return(retval);
		}

		sql_row=mysql_fetch_row(sql_res);
		retval=atoi(sql_row[0]);
		
		mysql_free_result(sql_res);
	
		return(retval);
	}

	return(retval);
}

